(****************************************************************************)
(*                                                                          *)
(*    Die Unit Star enthlt nur die Prozedur Stern. Diese Prozedur          *)
(*    berechnet nach Angabe von Startpositionen und Anfangsgeschwindigkeit  *)
(*    einer Anzahl von Sternen deren weitere Positionen entlang der         *)
(*    Zeitachse in der Ebene und zeichnet sie.                              *)
(*    Die Berechnung ergibt sich aus folgenden Vorraussetzungen fr den     *)
(*    Modellraum.                                                           *)
(*      -es gelten nur newtonsche Gesetze (keine relativ-Gesetze)           *)
(*      -alle Massen sind punktfrmig                                       *)
(*      -nur zweidimensionaler Raum                                         *)
(*      -Energieerhaltungssatz und Wechselwirkungsgesetze                   *)
(*                                                                          *)
(****************************************************************************)

Unit Star;

{$N+,E+}

interface

uses
  Graph,
  Dos,
  Drivers,
  Crt;

Type
  Werte = array[1..14,1..5] of real;   (* Globale Typenbezeichnung *)

Procedure Stern (Gen,maxit,Anz:longint; Sterne:werte; var Fehler:integer);

implementation

Procedure Stern (Gen,maxit,Anz:longint; Sterne:werte; var Fehler:integer);

const
  g=6.67e-11;                          (* Gravitationskonstante *)
  
type
  vektor = array[1..14,1..2] of double;
  masse  = array[1..14]      of double;
  schwer = array[1..2]       of double;

var
  xa,va,  xb,  xc,vc,  xd,xdiff,xf,vf   : vektor;
  m                                     : masse;
  SchwPunkt                             : schwer;
  k,z,u,i,j,n,gt,gm                     : integer;
  x,y,mitteX,mitteY                     : longint;
  Eps,h,hd,DeltaT,b,m_ges,grenze,ReX,ReY,
  genau,int,Hilfs_var,Dist_Xa,Dist_Xb   : double;

begin                                  (* Programm *)
  n:=Anz;
  for i:=1 to n do
   begin
    m[i]    :=Sterne[i,1];
    xa[i,1] :=Sterne[i,2];
    xa[i,2] :=Sterne[i,3];
    va[i,1] :=Sterne[i,4];
    va[i,2] :=Sterne[i,5];
   end;

  gt:=detect;                          (* Bildschirmrahmen festlegen *)
  initgraph(gt,gm,'bgi');
  mitteX:=getmaxx div 2;               (* Bildmitte *)
  mitteY:=getmaxy div 2;

  SchwPunkt[1]:=0;                     (* Schwerpunkt berechnen *)
  SchwPunkt[2]:=0;
  m_ges:=0;
  for i:=1 to n do
    begin
      for j:=1 to 2 do
        begin
          SchwPunkt[j]:=SchwPunkt[j]+m[i]*xa[i,j];
        end;
      m_ges:=m_ges+m[i];
    end;
  SchwPunkt[1]:=SchwPunkt[1]/m_ges;
  SchwPunkt[2]:=SchwPunkt[2]/m_ges;

  grenze:=0;                           (* Gre: real <=> Pixelabstand  *)
  for i:=1 to n do
    for j:=1 to 2 do
      begin
        Hilfs_Var := abs(xa[i,j]-SchwPunkt[j]);
        if Hilfs_Var > grenze then grenze := Hilfs_Var;
      end;
  int:=(grenze*2)/mitteY;
  genau:=1;
  for i:=1 to gen do
    genau:=genau*10;
  Eps:=grenze/Genau;
  DeltaT:=1;

repeat                                 (* gesamtes Mal- und Rechenprogramm *)

  for i:=1 to n do                     (* Pixel zeichnen *)
    begin
      ReX:=((xa[i,1]-SchwPunkt[1])/int)+mitteX;
      ReY:=((xa[i,2]-SchwPunkt[2])/int)+mitteY;
      if (abs(ReX)<2e9) and (abs(ReY)<2e9) then
        begin
          x:=round(ReX);
          y:=round(ReY);
          putpixel(x,y,(8+i));
        end
      else begin                       (* Abbruch bei Bereichsberschreitung *)
             closegraph;
             Fehler:=1;
             Exit;
           end;
    end;

  repeat                               (* Schrittweitensteuerung *)
    xf:=xa;
    vf:=va;

    for u:=1 to 3 do                   (* 3 Gnge fr neue Pos. X(DeltaT) *)
    begin                              (* 1 mit DeltaT, 2 mit DeltaT/2    *)

      for i:=1 to n do                 (* Startwerte fr Fixpunktit. X *)
        begin
          for j:=1 to 2 do
            xb[i,j] := xa[i,j]+ deltaT * va[i,j];
        end;

      z:=0;
      repeat                           (* Fixpunktit. fr X *)
        z:=z+1;
        b:=0;              
        for i:=1 to n do
          begin
            for j:=1 to 2 do
              begin
                xc[i,j]:= xa[i,j] + deltaT * va[i,j];
                for k:=1 to n do
                  if k<>i then begin
                    Dist_Xa:=sqrt(sqr(xa[k,1]-xa[i,1])+sqr(xa[k,2]-xa[i,2]));
                    Dist_Xb:=sqrt(sqr(xb[k,1]-xb[i,1])+sqr(xb[k,2]-xb[i,2]));
                    xc[i,j]:=xc[i,j]+  ((sqr(deltaT)*g*m[k])/2)*
                                       ((xb[k,j]-xb[i,j]+xa[k,j]-xa[i,j])/
                                       ((Dist_Xb+Dist_Xa)*(Dist_Xb*Dist_Xa)));
                  end;
                xDiff[i,j]:=xc[i,j]-xb[i,j];
                xb[i,j]:=xc[i,j];
              end;
            b:=b+abs(sqrt(sqr(xdiff[i,1])+sqr(xdiff[i,2])));
          end;
      until (z=maxit) or (b<(n*Eps));  (* Ende Fixpunktit. *)

      If u<>1 then begin               (* Berechnung von V *)
        for i:=1 to n do               
          for j:=1 to 2 do
            begin
              vc[i,j]:=va[i,j];
              for k:=1 to n do
                if k<>i then begin
                  Dist_Xa:=sqrt(sqr(xa[k,1]-xa[i,1])+sqr(xa[k,2]-xa[i,2]));
                  Dist_Xb:=sqrt(sqr(xb[k,1]-xb[i,1])+sqr(xb[k,2]-xb[i,2]));
                  vc[i,j]:=vc[i,j]+ (deltaT*g*m[k])*
                                    ((xb[k,j]-xb[i,j]+xa[k,j]-xa[i,j])/
                                    ((Dist_Xb+Dist_Xa)*(Dist_Xb*Dist_Xa)));
                end;
            end;
      end;

      if u=1 then                      (* halbieren von DeltaT fr *)
        begin                          (* 2. Durchgang             *)
          deltaT:=deltaT/2;
          xd:=xb;                      (* Werte fr Vergleich sichern *)
        end;

      if u=2 then                      (* neue Startwerte fr Berechnung *)
        begin                          (* von X(2*DeltaT/2) im 3. Gang   *)
          va:=vc;
          xa:=xb;
        end;
     
    end;                               (* Ende der 3 Durchgnge *)

    for i:=1 to n do                   (* Vergleich der 2 Werte fr X(DeltaT) *)
      for j:=1 to 2 do
        xdiff[i,j]:=xd[i,j]-xb[i,j];
    b:=0;
    for i:=1 to n do
      b:=b+abs(sqrt(sqr(xdiff[i,1])+sqr(xdiff[i,2])));

    hd:=sqrt(2*b/(n*Eps));             (* Rechnung fr Steuerung *)
    if hd<>0 then
      begin
        h:=2*deltaT/hd;
        deltaT:=2*h;
      end
    else begin                         (* Abbruch bei div. durch 0 *)
           closegraph;
           Fehler:=2;
           Exit;
         end;
    xa:=xf;
    va:=vf

  until hd<=2;                         (* Ende der Schrittweitensteuerung *)

  xa:=xb;
  va:=vc;

until  Keypressed;                     (* Abbruch fr Mal- und Rechenprogramm *)
      
closegraph;
end;                                   (* Ende SternProcedure *)
end.